@@ -6,7 +6,8 @@ our $VERSION = '0.975';
 
 use File::Temp ();
 use File::Spec;
-use Config ();
+use Cwd;
+use Config;
 
 =head1 NAME
 
@@ -48,21 +49,20 @@ sub write_parl {
     my $file = shift;
     if (not defined $file) {
         warn "${class}->write_parl() needs a file name as argument";
-        return();
+        return;
     }
 
     # write out to a temporary file first
     my ($fh, $tfile) = File::Temp::tempfile(
-        "parlXXXX", SUFFIX => $Config::Config{_exe}||'', TMPDIR => 1);
+        "parlXXXX", SUFFIX => $Config{_exe}||'', TMPDIR => 1, UNLINK => 1);
     close $fh;
-    # File::Temp, you suck!
 
     if (not $class->write_raw($tfile)) {
         unlink($tfile);
         warn "Could not write temporary parl (class $class) to file '$tfile'";
-        return();
+        return;
     }
-    chmod(oct('755'), $tfile);
+    chmod(0755, $tfile);
 
     # Use this to generate a real parl
     my @libs = ();
@@ -71,19 +71,12 @@ sub write_parl {
         $ilib =~ s/\\$/\\\\/;
         push(@libs, qq(-I$ilib) );
     }
-    my @args = (@libs, qw/-q -B/);
-
-    # prepend ./ if applicable
-    my $execute = $tfile;
-    my @dirs = File::Spec->splitdir(
-        (File::Spec->splitpath($execute))[1] # directory part only
-    );
-    $execute = File::Spec->catfile(File::Spec->curdir(), $tfile) if not @dirs;
-    system($execute, @args, "-O$file") and unlink($tfile), return();
-    
-    # clean up
-    unlink($tfile);
 
+    system(Cwd::abs_path($tfile), @libs, qw( -q -B ), "-O$file");
+    unless ($? == 0) {
+        warn "Failed to execute temporary parl (class $class) '$tfile': \$?=$?";
+        return;
+    }
     return 1;
 }
 
@@ -97,7 +90,7 @@ Returns the empty list on failure.
 
 sub get_raw {
     my $class = shift;
-    my $pos = $class->_data_pos(); 
+    my $pos = $class->_data_pos();
     if (not defined $pos) {
         warn "${class}->_data_pos() did not return the original tell() position of the DATA file handle";
         return();
@@ -128,22 +121,22 @@ Returns true on success and the empty list on failure.
 =cut
 
 sub write_raw {
-    my $class = shift;
-    my $file = shift;
+    my ($class, $file) = @_;
     if (not defined $file) {
         warn "${class}->write_raw() needs a file name as argument";
-        return();
+        return;
     }
     my $binary = $class->get_raw();
     if (not defined $binary) {
         warn "${class}->get_raw() did not return the raw binary data for a PAR loader";
-        return();
+        return;
     }
 
-    open my $fh, '>', $file or die "Could not open file '$file' for writing: $!";
-    binmode $fh;
-    print $fh $binary;
-    close $fh;
+    my $fh;
+    (open $fh, '>:raw', $file) 
+    && (print $fh $binary)
+    && (close $fh)
+        or die "Could not write file '$file': $!";
 
     return 1;
 }
@@ -162,7 +155,7 @@ Audrey Tang E<lt>cpan@audreyt.orgE<gt>
 
 Copyright 2006-2009 by Steffen Mueller E<lt>smueller@cpan.orgE<gt>.
 
-This program is free software; you can redistribute it and/or 
+This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 See F<LICENSE>.
